(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.cache = {}));
}(this, function (exports) { 'use strict';

  function argToKey() {
    for (var _len = arguments.length, arg = new Array(_len), _key = 0; _key < _len; _key++) {
      arg[_key] = arguments[_key];
    }

    return JSON.stringify(arg);
  }

  var cacheFunList = [];
  function push(func) {
    cacheFunList.push(func);
  }
  function clearCache() {
    cacheFunList.forEach(function (func) {
      func.clearCache();
    });
  }
  var store = {
    push: push,
    clearCache: clearCache
  };

  function promiseCache(fn) {
    var cache = {};

    var cachedFn = function cachedFn() {
      var key = argToKey.apply(void 0, arguments);
      return cache.hasOwnProperty(key) ? Promise.resolve(cache[key]) : fn.apply(void 0, arguments).then(function (res) {
        cache[key] = res;
        return Promise.resolve(res);
      });
    };

    cachedFn.clearCache = function (key) {
      if (key) {
        delete cache[key];
      } else {
        cache = {};
      }
    };

    cachedFn.funType = 'promise';
    cachedFn.cacheType = 'memory';
    cachedFn.originFunName = fn.name;
    store.push(cachedFn);
    return cachedFn;
  }

  /**
   * @module Storage
   * @author crossjs <liwenfu@crossjs.com>
   */
  /**
   * @param {string} prefix  key 前缀
   * @param {number} expire  默认过期秒数
   */

  function Storage(prefix, expire) {
    this.prefix = prefix || '';

    if (expire === -1) {
      this.driver = window.sessionStorage;
    } else {
      this.driver = window.localStorage;
      this.expire = expire || 0;
    }
  }

  Storage.prototype = {
    constructor: Storage,
    _key: function (key) {
      return this.prefix + key;
    },

    /**
     * 获取所有的本地存储数据对应的 key
     */
    keys: function () {
      var keys = Object.keys(this.driver);

      if (this.prefix) {
        var index = this.prefix.length;
        return keys.map(function (key) {
          return key.substring(index);
        });
      }

      return keys;
    },

    /**
     * 移除某一项本地存储的数据
     */
    remove: function (key) {
      this.driver.removeItem(this._key(key));
    },

    /**
     * 清除所有本地存储的数据
     */
    clear: function () {
      this.driver.clear();
    },

    /**
     * 将数据进行本地存储
     */
    set: function (key, value, expire) {
      var data = {
        value: value
      };

      if (typeof expire === 'undefined') {
        expire = this.expire;
      }

      if (expire) {
        data.expire = Date.now() + expire * 1000;
      }

      this.driver.setItem(this._key(key), JSON.stringify(data));
    },

    /**
     * 提取本地存储的数据
     */
    get: function (key) {
      var data = this.driver.getItem(this._key(key));

      if (data) {
        data = JSON.parse(data);

        if (data.expire) {
          if (data.expire < Date.now()) {
            this.remove(key);
            data = null;
          }
        }
      }

      return data && data.value;
    }
  };
  var ndStorage = Storage;

  var SessionStorage = new ndStorage('S-CACHE-', -1);

  function promiseSessionStorageCache(fn, storageKey) {
    var cache = SessionStorage.get(storageKey) || {};

    var cachedFn = function cachedFn() {
      var key = argToKey.apply(void 0, arguments);
      return cache.hasOwnProperty(key) ? Promise.resolve(cache[key]) : fn.apply(void 0, arguments).then(function (res) {
        cache[key] = res;
        SessionStorage.set(storageKey, cache);
        return Promise.resolve(res);
      });
    };

    cachedFn.clearCache = function (key) {
      if (key) {
        delete cache[key];
        SessionStorage.set(storageKey, cache);
      } else {
        cache = {};
        SessionStorage.remove(storageKey);
      }
    };

    cachedFn.funType = 'promise';
    cachedFn.cacheType = 'sessionStorage';
    cachedFn.cacheKey = storageKey;
    cachedFn.originFunName = fn.name;
    store.push(cachedFn);
    return cachedFn;
  }

  function pureFuncCache(fn) {
    var cache = {};

    var cachedFn = function cachedFn() {
      var cacheKey = argToKey.apply(void 0, arguments);
      return cache.hasOwnProperty(cacheKey) ? cache[cacheKey] : cache[cacheKey] = fn.apply(void 0, arguments);
    };

    cachedFn.clearCache = function (key) {
      if (key) {
        delete cache[key];
      } else {
        cache = {};
      }
    };

    cachedFn.funType = 'pureFunc';
    cachedFn.cacheType = 'memory';
    cachedFn.originFunName = fn.name;
    store.push(cachedFn);
    return cachedFn;
  }

  exports.argToKey = argToKey;
  exports.clearCache = clearCache;
  exports.promiseMemoryCache = promiseCache;
  exports.promiseSessionStorageCache = promiseSessionStorageCache;
  exports.pureFuncMemoryCache = pureFuncCache;

}));
//# sourceMappingURL=cache.js.map
