"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = promiseSessionStorageCache;

var _store = _interopRequireDefault(require("./store"));

var _storage = require("../utils/storage");

var _assist = require("../utils/assist");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 异步的结果缓存在session storage中，下次调用直接使用缓存数据
 * @param {Function} fn - 返回值是Promise的函数
 * @param {Function} storageKey - 缓存在session storage中的key前缀
 * @returns {Function} 和原函数的调用方式是一样的，第一次调用数据缓存后，之后的调用不会调用原函数，而是会直接使用缓存后的数据，另外该函数还带有一个静态的clearCache方法，可以用来清空缓存数据
 * @property {Function} clearCache 清空缓存数据
 */
function promiseSessionStorageCache(fn, storageKey) {
  var cache = _storage.SessionStorage.get(storageKey) || {};

  var cachedFn = function cachedFn() {
    var key = _assist.argToKey.apply(void 0, arguments);

    return cache.hasOwnProperty(key) ? Promise.resolve(cache[key]) : fn.apply(void 0, arguments).then(function (res) {
      cache[key] = res;

      _storage.SessionStorage.set(storageKey, cache);

      return Promise.resolve(res);
    });
  };

  cachedFn.clearCache = function (key) {
    if (key) {
      delete cache[key];

      _storage.SessionStorage.set(storageKey, cache);
    } else {
      cache = {};

      _storage.SessionStorage.remove(storageKey);
    }
  };

  cachedFn.funType = 'promise';
  cachedFn.cacheType = 'sessionStorage';
  cachedFn.cacheKey = storageKey;
  cachedFn.originFunName = fn.name;

  _store.default.push(cachedFn);

  return cachedFn;
}